import bb.cascades 1.2
import bb.device 1.2

Container {
    id: container
    property string letter
    property int keyWidth
    property int keyHeight: touchKeyboardContainer.eachKeyHeight
    property bool isShiftKeyAndPressed: ((letter == "↑") && (touchKeyboardContainer.shiftPressedOnce || touchKeyboardContainer.shiftLongPressed))
    property bool touched: false
    property bool keyWasLongPressedSkipHandleTap: false
    
    onTouchedChanged: {
//        if (touched)
//            console.log("Touched!!!! " + letter)
    }

    onIsShiftKeyAndPressedChanged: {
        if (isShiftKeyAndPressed)
            label.textStyle.color = touchKeyboardContainer.coloredLabelColor
        else 
            label.textStyle.color = touchKeyboardContainer.defaultLabelColor
    }   
         
    maxWidth: keyWidth
    maxHeight: keyHeight
    minWidth: maxWidth
    minHeight: maxHeight
    visible: (letter != "")
    layout: DockLayout {}
    onTouch: {
        if (event.touchType == TouchType.Up) {
            handleTap()
            hapticFeedback.visible = false
        }
        else {
//            hapticFeedback.translationX = event.windowX - (hapticFeedback.minWidth / 2)
//            hapticFeedback.translationY = event.windowY - (hapticFeedback.minHeight)
//            hapticFeedback.visible = true
        }
    }
    gestureHandlers: [
        LongPressHandler {
            onLongPressed: { 
                touchKeyboardContainer.handleLongPressed(letter) 
                keyWasLongPressedSkipHandleTap = true
            }
        },
        TapHandler {
            id: tapHandler
            onTapped: {
//                handleTap()
            }
        }
    ]
        
    attachedObjects: [
        TextStyleDefinition {
            id: urduStyle
            //            base: SystemDefaults.TextStyles.TitleText
            rules: [
                FontFaceRule {
                    source: "asset:///Fonts/Nazanin.ttf"
                    fontFamily: "Gotham-Medium"
                }
            ]
            fontFamily: "Gotham-Medium, sans-serif"
        },
        VibrationController { id: vibrationController }
    ]
    
    ImageView {
        id: imageView
        imageSource: {
            if (mainPage.theme == "dark") {
                if ((touched) || ((letter == "↑") && touchKeyboardContainer.shiftLongPressed) || ((letter == "@123") && touchKeyboardContainer.a123LongPressed)) {
                    "asset:///images/keyButtons/" + touchKeyboardContainer.buttonType + "/white" + touchKeyboard.buttonType + "Pressed.amd"
                } else {
                    "asset:///images/keyButtons/" + touchKeyboardContainer.buttonType + "/white" + touchKeyboard.buttonType + ".amd"
                }
            } else {
                if ((touched) || ((letter == "↑") && touchKeyboardContainer.shiftLongPressed) || ((letter == "@123") && touchKeyboardContainer.a123LongPressed)) {
                    "asset:///images/keyButtons/" + touchKeyboardContainer.buttonType + "/black" + touchKeyboard.buttonType + "Pressed.amd"
             } else {
                 "asset:///images/keyButtons/" + touchKeyboardContainer.buttonType + "/black" + touchKeyboard.buttonType + ".amd"
                }
            }
        }
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill
        scalingMethod: ScalingMethod.Fill
        onTouch: {
            if ((event.touchType == TouchType.Down) || (event.touchType == TouchType.Move)) 
                touched = true
            else 
                touched = false
        }
    }

    Label {
        id: label
        text: (letter == "ONE_KEY_SPACE") ? " " : ((touchKeyboardContainer.shiftPressedOnce || touchKeyboardContainer.shiftLongPressed) ? letter.toUpperCase() : letter.toLowerCase())
        visible: !((letter.toLowerCase() == "@123") || (letter.toLowerCase() == "copy") || (letter.toLowerCase() == "share"))
        horizontalAlignment: HorizontalAlignment.Center
        verticalAlignment: VerticalAlignment.Center
        textStyle.fontSize: FontSize.PointValue
        textStyle.fontSizeValue: ((((keyHeight <= keyWidth) ? (keyHeight / 10) : (keyWidth / 10))) * ((letter == "تومان") ? 0.75 : 1)) * (touchKeyboardContainer.userPreferenceLetterSize)
        textStyle.color: ((letter == "↵") || (letter == "@123") || (letter == "🌐") || (letter == "COPY") || (letter == "SHARE") || (letter == "←")) ? touchKeyboardContainer.coloredLabelColor : touchKeyboardContainer.defaultLabelColor
        textStyle.base: textArea.textStyle.base
        rotationZ: (letter == "تومان") ? 315 : 0
        content { flags: TextContentFlag.Emoticons }
        onTouch: {
            if ((event.touchType == TouchType.Down) || (event.touchType == TouchType.Move)) 
                touched = true
            else 
                touched = false
        }
    }
    ImageView {
        visible: (!label.visible && (letter != "@123"))
        enabled: !mainPage.textIsEmpty
        opacity: enabled ? 1 : 0.3
        imageSource: {
            if (letter == "SHARE") {
                if (mainPage.theme == "dark") 
                    "asset:///images/ic_share.png"
                else 
                    "asset:///images/ic_share_black.png"
            }             
            else if (letter == "COPY") {
                if (mainPage.theme == "dark") 
                    "asset:///images/ic_copy.png"
                else 
                    "asset:///images/ic_copy_black.png"                    
            }
        }
        horizontalAlignment: HorizontalAlignment.Center
        verticalAlignment: VerticalAlignment.Center
        maxHeight: container.keyHeight
        maxWidth: container.keyWidth
        scalingMethod: ScalingMethod.AspectFit
        onTouch: {
            if (enabled) {
                if ((event.touchType == TouchType.Down) || (event.touchType == TouchType.Move)) 
                    touched = true
                else 
                    touched = false
            }                   
        }                   
    }
    Label {
        text: letter
        visible: (letter == "@123")
        horizontalAlignment: HorizontalAlignment.Center
        verticalAlignment: VerticalAlignment.Center
        textStyle.fontSize: FontSize.PointValue
        //            textStyle.fontSizeValue: (keyHeight / 10) * 0.5
        textStyle.fontSizeValue: ((keyHeight <= keyWidth) ? (keyHeight / 10) : (keyWidth / 10)) * 0.5
        rotationZ: 315
        textStyle.color: enabled ? touchKeyboardContainer.coloredLabelColor : Color.Gray
        opacity: enabled ? 1 : 0.5
        content { flags: TextContentFlag.Emoticons }
        onTouch: {
            if (enabled) {
                if ((event.touchType == TouchType.Down) || (event.touchType == TouchType.Move)) 
                    touched = true
                else 
                    touched = false
            }                   
        }
    }

    function handleTap() {
        if (keyWasLongPressedSkipHandleTap) {
            keyWasLongPressedSkipHandleTap = false
            return
        } else {
            displayedLongPressLettersContainer.visible = false
            lastLetterLongPressed = false 
        }
        if (touchKeyboardContainer.userIsSwiping) {
            // We want to reject the tap if the user was swiping
            touchKeyboardContainer.userIsSwiping = false
            return 
        }
        if (letter == "←") {
            textArea.editor.setSelection(textArea.editor.cursorPosition - 1, textArea.editor.cursorPosition)
            textArea.editor.insertPlainText("")
        }
        else if (letter == "🌐") {
            if (touchKeyboardContainer.keyboardLayout != 0)
                touchKeyboardContainer.keyboardLayout = 0
                
            var indexOfNextKeyboard = 0
            if (touchKeyboardContainer.listOfKeyboardsEnabled.length == 0) {
                var defaultEnableKeyboard = new Array("QWERTY")
                touchKeyboardContainer.listOfKeyboardsEnabled = defaultEnableKeyboard
            }
            for (var i = 0; i < touchKeyboardContainer.listOfKeyboardsEnabled.length; i++) {
                if (touchKeyboardContainer.listOfKeyboardsEnabled[i] == touchKeyboardContainer.keyboardLanguage)
                    indexOfNextKeyboard = (i < (touchKeyboardContainer.listOfKeyboardsEnabled.length - 1)) ? (i + 1) : 0
            }
            touchKeyboardContainer.keyboardLanguage = touchKeyboardContainer.listOfKeyboardsEnabled[indexOfNextKeyboard]
        }
        else if (letter == "COPY") {
            if (!mainPage.textIsEmpty) {
                if (_app.saveToClipboard(textArea.text)) {
                    systemToast.body = "Text copied to clipboard successfully!"
                    systemToast.show()
                    if (mainPage.isCard)
                        _app.cardDone(qsTr("Content Shared."));
                }
            }
        }
        else if (letter == "SHARE") {
            if (!mainPage.textIsEmpty) {
                /*
                shareInvoke.query.mimeType = "text/plain"
                shareInvoke.query.data = textArea.text
//                shareInvoke.query.data = _app.handleMultipleLanguageSharing(textArea.text)
                shareInvoke.query.updateQuery()
                shareInvoke.trigger("bb.action.SHARE");
                */
                _app.shareWithInvocation(textArea.text)
            }
        }
        else if (letter == "↑") {
            if (touchKeyboardContainer.shiftLongPressed) {
                touchKeyboardContainer.shiftPressedOnce = false 
                touchKeyboardContainer.shiftLongPressed = false
            }
            else 
                touchKeyboardContainer.shiftPressedOnce = !touchKeyboardContainer.shiftPressedOnce
        }
        else if (letter == "@123") {
            if (touchKeyboardContainer.a123LongPressed) {
                touchKeyboardContainer.a123LongPressed = false
                touchKeyboardContainer.keyboardLayout = 0
            }
            else {
                touchKeyboardContainer.keyboardLayout += 1
                if (touchKeyboardContainer.keyboardLayout > 2)
                    touchKeyboardContainer.keyboardLayout = 0
            }
        }
        else {
            if (letter == "↵") {
                if (touchKeyboardContainer.shiftPressedOnce)
                    touchKeyboardContainer.shiftPressedOnce = false
                textArea.editor.insertPlainText("\n")
            }
            else {
                if ((letter == "ONE_KEY_SPACE") || (letter == " "))
                    textArea.editor.insertPlainText(" ")
                else {
                    // Insert the letter pressed
                    textArea.editor.insertPlainText(((touchKeyboardContainer.shiftPressedOnce || touchKeyboardContainer.shiftLongPressed) ? letter.toUpperCase() : letter.toLowerCase()))
                    
//                    if (_settings.value("hapticFeedbackEnabled", false)) {
//                        vibrationController.start(1, 1)
//                    }
                    
                    if (touchKeyboardContainer.shiftPressedOnce)
                        touchKeyboardContainer.shiftPressedOnce = false
                        
                    if ((touchKeyboardContainer.keyboardLayout > 0) && (!touchKeyboardContainer.a123LongPressed))
                        touchKeyboardContainer.keyboardLayout = 0
                }
            }
            
            if ((letter == " ") || (letter == "ONE_KEY_SPACE")) {
                if ((textArea.text.substring(textArea.text.length - 2, textArea.text.length) == ". ") || (textArea.text.substring(textArea.text.length - 2, textArea.text.length) == "! ") || (textArea.text.substring(textArea.text.length - 2, textArea.text.length) == "? "))
                    touchKeyboardContainer.shiftPressedOnce = true
                else 
                    touchKeyboardContainer.shiftPressedOnce = false
            }
        }
    }
}
