import bb.cascades 1.2

Sheet {
    id: tutorialSheet
    onOpened: {
        listView.scrollToItem([0], ScrollAnimation.Smooth)
    }
    onClosed: {
        settingsSheet.open()
    }
    
    Page {
        id: tutorialPage
        titleBar: TitleBar {
            id: titleBar
            title: qsTr("Tutorial")
            acceptAction: ActionItem {
                title: tutorialPage.pageShown == 5 ? qsTr("Done") : qsTr("Next")
                onTriggered: {
                    if (tutorialPage.pageShown == 5) {
                        if (_settings.value("showFirstTimeTutorial", true))
                            _settings.setValue("showFirstTimeTutorial", false)
                            
                        tutorialSheet.close()
                    }
                    else 
                        listView.scrollToItem([tutorialPage.pageShown + 1], ScrollAnimation.Smooth)
                }
            }
            dismissAction: ActionItem {
                title: qsTr("Dismiss")
                onTriggered: {
                    tutorialSheet.close()
                }
            }
        }
                
        property int pageShown
        
        Container {
            id: tutorialContainer
            layout: DockLayout {}
            horizontalAlignment: HorizontalAlignment.Fill
            verticalAlignment: VerticalAlignment.Fill
            background: Color.DarkGray
            ListView {
                id: listView
                
                property int deviceWidth: mainPage.deviceWidth
                property int deviceHeight: mainPage.deviceHeight 
                property variant background: tutorialContainer.background
                
                dataModel: XmlDataModel { source: "asset:///Tutorial/tutorialModel.xml" }
                layout: StackListLayout { orientation: LayoutOrientation.LeftToRight }
                flickMode: FlickMode.SingleItem
                
                function updatePageShown(page) {
                    tutorialPage.pageShown = page
                }
                
                listItemComponents: [
                    ListItemComponent {
                        type: "item"
                        Container {
                            id: itemContainer
                            layout: DockLayout {}
                            preferredWidth: itemContainer.ListItem.view.deviceWidth
                            preferredHeight: itemContainer.ListItem.view.deviceHeight
                            attachedObjects: LayoutUpdateHandler {
                                id: luhPosition
                                onLayoutFrameChanged: {
                                    if (luhPosition.layoutFrame.x >= - itemContainer.preferredWidth / 2 && luhPosition.layoutFrame.x <= itemContainer.preferredWidth / 2) {
                                        itemContainer.ListItem.view.updatePageShown(ListItemData.page)
                                    } 
                                }
                            }
                            Container {
                                layout: DockLayout {}
                                horizontalAlignment: HorizontalAlignment.Fill
                                verticalAlignment: VerticalAlignment.Fill
                                Container {
                                    topPadding: itemContainer.ListItem.view.deviceHeight * 0.05
                                    bottomPadding: topPadding
                                    leftPadding: itemContainer.ListItem.view.deviceWidth * 0.05
                                    rightPadding: leftPadding
                                    horizontalAlignment: HorizontalAlignment.Center
                                    verticalAlignment: VerticalAlignment.Center
                                    ImageView {
                                        imageSource: ListItemData.image
                                        maxHeight: itemContainer.ListItem.view.deviceHeight * 0.7
                                        maxWidth: itemContainer.ListItem.view.deviceWidth * 0.7
                                        scalingMethod: ScalingMethod.AspectFit
                                    }
                                }
                                Container {
                                    topPadding: itemContainer.ListItem.view.deviceHeight * 0.05
                                    bottomPadding: topPadding
                                    leftPadding: itemContainer.ListItem.view.deviceWidth * 0.05
                                    rightPadding: leftPadding
                                    horizontalAlignment: HorizontalAlignment.Fill
                                    verticalAlignment: VerticalAlignment.Top
                                    opacity: 0.9
                                    background: itemContainer.ListItem.view.background
                                    Label {
                                        text: ListItemData.description
                                        multiline: true
                                        horizontalAlignment: HorizontalAlignment.Center
                                        textStyle.color: Color.White
                                    }
                                }
                            }
                        }
                    }
                ]    
            }
        }
    }
}
