import bb.cascades 1.2
import bb.system 1.2
import Roger.CustomTimer 1.0

Page {
    id: mainPage

    property string theme: _settings.value("theme", "dark")
    property int deviceWidth
    property int deviceHeight
    property int textAreaFontSize: _settings.value("textAreaFontSize", 8)
    property bool keepFocusEnabled: true
    property int paddingForTitleBarInCard: (_ui.du(15))
    property string textBeforeDeleteText
    property bool textIsEmpty: true
    property bool isCard: true
    
    onDeviceHeightChanged: {
        console.log("card.qml->onDeviceHeightChanged : " + deviceHeight)
        var layout = touchKeyboard.keyboardLanguage
        touchKeyboard.keyboardIsResetting = true 
        touchKeyboard.keyboardLanguage = "Reset"
        touchKeyboard.keyboardLanguage = layout
        touchKeyboard.keyboardIsResetting = false
    }
    
    onKeepFocusEnabledChanged: { if (keepFocusEnabled) textArea.requestFocus() }

    onCreationCompleted: { _app.initialCardTextChanged.connect(onInitialCardTextChanged) }
    function onInitialCardTextChanged(text) { textArea.text = text }
    
    titleBar: TitleBar {
        title: qsTr("Silent Keys")
        acceptAction: ActionItem {
            title: qsTr("Copy Text")
            onTriggered: {
                _app.saveToClipboard(textArea.text)
                _app.cardDone(qsTr("Content Shared."));
            }
        }
        dismissAction: ActionItem {
            title: qsTr("Cancel")
            onTriggered: {
                _app.cardDone(qsTr("Dont feel like sharing today?"));
            }
        }
    }
    
    attachedObjects: [
        SystemToast {
            id: systemToast
            position: SystemUiPosition.TopCenter
            button.label: "Ok"
        },
        SystemToast {
            id: deleteTextSystemToast
            button.label: "Undo"
            button.enabled: true            
            onFinished: {
                if (buttonSelection().label == "Undo")
                    textArea.text = textBeforeDeleteText
            }
        },
        Timer {
            id: timer
            interval: 3000
            onTimeout: { deleteTextSystemToast.cancel(); timer.stop() }
        },
        Invocation {
            id: shareInvoke
            onArmed: {
                trigger("bb.action.SHARE")
            }
        },
        RenderFence { 
            raised: true
            onReached: { 
                textArea.requestFocus()
                touchKeyboard.keyboardIsResetting = false 
            }
        }
    ]
    
    Container {
        layout: DockLayout {}
        DeviceWidthAndHeightSelector {}
        Container {
            layout: StackLayout { orientation: LayoutOrientation.LeftToRight }
            TextArea {
                id: textArea
                text: ""
                onTextChanging: {
                    if (text == "") 
                        textIsEmpty = true
                    else    
                        textIsEmpty = false
                                        
                    if ((text == "") || (text.substring(text.length - 1, text.length) == "\n") || (text.substring(text.length - 2, text.length) == ". ") || (text.substring(text.length - 2, text.length) == "! ") || (text.substring(text.length - 2, text.length) == "? "))
                        touchKeyboard.shiftPressedOnce = true
                    else 
                        touchKeyboard.shiftPressedOnce = false
                }
                verticalAlignment: VerticalAlignment.Top
                maxHeight: deviceHeight - (touchKeyboard.eachKeyHeight * 4) - (touchKeyboard.paddingBetweenRows * 4) - paddingForTitleBarInCard
                textStyle.fontSize: FontSize.PointValue
                textStyle.fontSizeValue: textAreaFontSize
                onFocusedChanged: { if ((!focused) && (keepFocusEnabled)) requestFocus() }
                content { flags: TextContentFlag.Emoticons | TextContentFlag.ActiveText }
                input { flags: touchKeyboard.keyboardLanguage == "Special_Characters" ? TextInputFlag.AutoCorrection | TextInputFlag.AutoCapitalization | TextInputFlag.SpellCheck | TextInputFlag.WordSubstitution : TextInputFlag.VirtualKeyboardOff | TextInputFlag.AutoCorrection | TextInputFlag.AutoCapitalization | TextInputFlag.SpellCheck | TextInputFlag.WordSubstitution }
                onCreationCompleted: { requestFocus(); }
            }
            Container {
                topPadding: _ui.du(1.6)
                ImageView {
                    id: deleteImage
                    visible: !textIsEmpty
                    imageSource: (theme == "dark") ? "asset:///images/ic_delete_white.png" : "asset:///images/ic_delete_black.png"
                    maxHeight: _ui.du(5)
                    maxWidth: maxHeight
                    onTouch: {
                        if (event.touchType == TouchType.Up) {
                            textBeforeDeleteText = textArea.text
                            textArea.text = ""
                            
                            deleteTextSystemToast.cancel()
                            deleteTextSystemToast.body = "Deleted all text"
                            deleteTextSystemToast.show()
                            
                            timer.start()
                        }
                    }
                }
            }
        }
        TouchKeyboard {
            id: touchKeyboard
            verticalAlignment: VerticalAlignment.Bottom            
        }
    }
}
