/*
 * Copyright (c) 2011-2014 BlackBerry Limited.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import bb.cascades 1.2
import bb.system 1.2
import Roger.CustomTimer 1.0
import "Tutorial"

Page {
    id: mainPage
    
    property string theme: _settings.value("theme", "dark")
    property int deviceWidth: _settings.value("deviceWidth", 720)
    property int deviceHeight: _settings.value("deviceHeight", 720)
    property int textAreaFontSize: _settings.value("textAreaFontSize", 8)
    property bool keepFocusEnabled: true
    property int paddingForTitleBarInCard: 0
    property string textBeforeDeleteText
    property bool textIsEmpty: true
    property bool isCard: false
    
    onDeviceHeightChanged: {
        console.log("main.qml->onDeviceHeightChanged : " + deviceHeight)
        var layout = touchKeyboard.keyboardLanguage
        touchKeyboard.keyboardIsResetting = true 
        touchKeyboard.keyboardLanguage = "Reset"
        touchKeyboard.keyboardLanguage = layout
        touchKeyboard.keyboardIsResetting = false
    }

    onKeepFocusEnabledChanged: { if (keepFocusEnabled) textArea.requestFocus() }

    Menu.definition: MenuDefinition {
        settingsAction: [
            SettingsActionItem {
                ActionBar.placement: ActionBarPlacement.OnBar
                onTriggered: {
                    settingsSheet.open()
                }
            }
        ]
        helpAction: [
            HelpActionItem {
                ActionBar.placement: ActionBarPlacement.OnBar
                onTriggered: {
                    helpSheet.open()
                }
            }
        ]
        actions: [
            ActionItem {
                title: qsTr("About")
                ActionBar.placement: ActionBarPlacement.OnBar
                imageSource: "asset:///images/ic_info.png"
                onTriggered: {
                    aboutSheet.open()
                }
            }
        ]      
    } // end of MenuDefinition
    
    attachedObjects: [
        AboutSheet {
            id: aboutSheet
            onOpened: keepFocusEnabled = false
            onClosed: keepFocusEnabled = true
        },
        HelpSheet {
            id: helpSheet
            onOpened: keepFocusEnabled = false
            onClosed: keepFocusEnabled = true
        },
        SettingsSheet {
            id: settingsSheet
            onOpened: keepFocusEnabled = false
            onClosed: keepFocusEnabled = true
        },
        ComponentDefinition {
            id: menu
        },
        ComponentDefinition { 
            id: changelogContainer 
        },
        Invocation {
            id: invoke
            query {
                invokeTargetId: "sys.appworld"
            }
            onArmed: {
                trigger("bb.action.OPEN")
            }
        },
        Invocation {
            id: shareInvoke
            onArmed: {
                trigger("bb.action.SHARE")
            }
        },
        SystemToast {
            id: systemToast
            position: SystemUiPosition.TopCenter
            button.label: "Ok"
        },
        SystemToast {
            id: deleteTextSystemToast
            button.label: "Undo"
            button.enabled: true            
            onFinished: {
                if (buttonSelection().label == "Undo")
                    textArea.text = textBeforeDeleteText
            }
        },
        Timer {
            id: timer
            interval: 3000
            onTimeout: { deleteTextSystemToast.cancel(); timer.stop() }
        },
        AppCover { id: activeFrame },
        RenderFence { 
            raised: true
            onReached: { 
                textArea.requestFocus()
                touchKeyboard.keyboardIsResetting = false 
            }
        },
        TutorialSheet { id: tutorialSheet }
    ]

    onCreationCompleted: {
        // Creates a MultiCover or SceneCover depending on asset selection
        Application.setCover(activeFrame)
        
//        Application.manualExit.connect(exitApp);
//        Application.setAutoExit(false);

        if (_settings.value("showFirstTimeTutorial", true))
            tutorialSheet.open()
            
        console.log("1")
        _app.newChangelog.connect(onNewChangelog)
        console.log("2")
    }
    
    function exitApp()
    {
        /* This is needed to stop the application generating a 
         "application has been slayed" error on close. Should
         not be needed on final release of BB10!!
         */
        Application.setCover(0);
                
        Application.quit();   
    } 

    function onNewChangelog(newChangelogForThisUser) {
        console.log("3")
        changelogContainer.source = "ChangelogContainer.qml"
        var thisChangelogContainer = changelogContainer.createObject()
        thisChangelogContainer.changelogText = "Changelog:\n" + newChangelogForThisUser
        thisChangelogContainer.changelogDone.connect(onChangelogDone)
        mainContainer.add(thisChangelogContainer)
        console.log("4")
    }

    function onChangelogDone(thisChangelogContainer) {
        mainContainer.remove(thisChangelogContainer)
        console.log("5")
    }
    
    Container {
        id: mainContainer
        layout: DockLayout {}
//        DeviceWidthAndHeightSelector {}
        Container {
            layout: StackLayout { orientation: LayoutOrientation.LeftToRight }
            TextArea {
                id: textArea
                text: ""
                onTextChanging: {
                    if (text == "") 
                        textIsEmpty = true
                    else    
                        textIsEmpty = false
                    
                    if ((text == "") || (text.substring(text.length - 1, text.length) == "\n") || (text.substring(text.length - 2, text.length) == ". ") || (text.substring(text.length - 2, text.length) == "! ") || (text.substring(text.length - 2, text.length) == "? "))
                        touchKeyboard.shiftPressedOnce = true
                    else 
                        touchKeyboard.shiftPressedOnce = false
                }
                verticalAlignment: VerticalAlignment.Top
                maxHeight: deviceHeight - (touchKeyboard.eachKeyHeight * 4) - (touchKeyboard.paddingBetweenRows * 4) - paddingForTitleBarInCard
                textStyle.fontSize: FontSize.PointValue
                textStyle.fontSizeValue: textAreaFontSize
                onFocusedChanged: { if ((!focused) && (keepFocusEnabled)) requestFocus() }
                content { flags: TextContentFlag.Emoticons | TextContentFlag.ActiveText }
                input { flags: touchKeyboard.keyboardLanguage == "Special_Characters" ? TextInputFlag.AutoCorrection | TextInputFlag.AutoCapitalization | TextInputFlag.SpellCheck | TextInputFlag.WordSubstitution : TextInputFlag.VirtualKeyboardOff | TextInputFlag.AutoCorrection | TextInputFlag.AutoCapitalization | TextInputFlag.SpellCheck | TextInputFlag.WordSubstitution }
                onCreationCompleted: { requestFocus(); }
            }
            Container {
                topPadding: _ui.du(1.6)
                ImageView {
                    id: deleteImage
                    visible: !textIsEmpty
                    imageSource: (theme == "dark") ? "asset:///images/ic_delete_white.png" : "asset:///images/ic_delete_black.png"
                    maxHeight: _ui.du(5)
                    maxWidth: maxHeight
                    onTouch: {
                        if (event.touchType == TouchType.Up) {
                            textBeforeDeleteText = textArea.text
                            textArea.text = ""
                            
                            deleteTextSystemToast.cancel()
                            deleteTextSystemToast.body = "Deleted all text"
                            deleteTextSystemToast.show()
                            
                            timer.start()
                        }
                    }
                }
            }
        }
        TouchKeyboard {
            id: touchKeyboard
            verticalAlignment: VerticalAlignment.Bottom            
        }
        HapticFeedback { id: hapticFeedback }
    }
}
